#include "wiimote.h"

void wiimote_init(int argc, char **argv) {
  bdaddr_t bdaddr;
  if (argc > 1) {
    str2ba(argv[1], &bdaddr);
  }
  else {
    bdaddr = *BDADDR_ANY;
  }

  printf("Put Wiimote in discoverable mode now (press 1+2)...\n");
  if (!(wiimote = cwiid_open(&bdaddr, 0))) {
    fprintf(stderr, "Unable to connect to wiimote\n");
    exit(-1);
  }
  wiimote_update();
  printf("Battery: %i%%\n", state.battery);
/*   printf("Battery: %i%%\n", (int)(100.0 * state.battery / CWIID_BATTERY_MAX)); */

  cwiid_get_acc_cal(wiimote, CWIID_EXT_NONE, &cal);

  toggle_bit(led_state, CWIID_LED1_ON);
  set_led_state(wiimote, led_state);

  toggle_bit(rpt_mode, CWIID_RPT_BTN);
  set_rpt_mode(wiimote, rpt_mode);

  toggle_bit(rpt_mode, CWIID_RPT_ACC);
  set_rpt_mode(wiimote, rpt_mode);

}

void wiimote_update(void) {
  cwiid_get_state(wiimote, &state);
}

int up(void) {
/*   wiimote_update(); */
/*   return (state.buttons & CWIID_BTN_B); */

  wiimote_move();
  return (wii_move.pitch < -0.3);
}

int right(void) {
/*   wiimote_update(); */
/*   return (state.buttons & CWIID_BTN_RIGHT); */

  wiimote_move();
  return (wii_move.roll < -0.3);
}

int left(void) {
/*   wiimote_update(); */
/*   return (state.buttons & CWIID_BTN_LEFT); */

  wiimote_move();
  return (wii_move.roll > 0.3);
}

void set_led_state(cwiid_wiimote_t *wiimote, unsigned char led_state)
{
    if (cwiid_set_led(wiimote, led_state)) {
        fprintf(stderr, "Error setting LEDs \n");
    }
}

void set_rpt_mode(cwiid_wiimote_t *wiimote, unsigned char rpt_mode)
{
    if (cwiid_set_rpt_mode(wiimote, rpt_mode)) {
        fprintf(stderr, "Error setting report mode\n");
    }
}

void wiimote_move() {
  double a_x, a_y, a_z, a;
  double roll, pitch;

  wiimote_update();

  a_x = ((double)state.acc[CWIID_X] - cal.zero[CWIID_X]) /
    (cal.one[CWIID_X] - cal.zero[CWIID_X]);
  a_y = ((double)state.acc[CWIID_Y] - cal.zero[CWIID_Y]) /
    (cal.one[CWIID_Y] - cal.zero[CWIID_Y]);
  a_z = ((double)state.acc[CWIID_Z] - cal.zero[CWIID_Z]) /
    (cal.one[CWIID_Z] - cal.zero[CWIID_Z]);
  a = sqrt(pow(a_x,2)+pow(a_y,2)+pow(a_z,2));

  roll = atan(a_x/a_z);
  if (a_z <= 0.0) {
    roll += M_PI * ((a_x > 0.0) ? 1 : -1);
  }
  roll *= -1;

  pitch = atan(a_y/a_z*cos(roll));

  wii_move.acc_x = a_x;
  wii_move.acc_y = a_y;
  wii_move.acc_z = a_z;
  wii_move.acc = a;
  wii_move.roll = roll;
  wii_move.pitch = pitch;
}
